﻿namespace UltraGridSelectGroupedRows
{
    using System;
    using System.Data;
    using System.Windows.Forms;
    using Infragistics.Win;
    using Infragistics.Win.UltraWinGrid;

    public partial class Form1 : Form
    {
        private UIElement _uiElement;
        private Infragistics.Win.UltraWinGrid.ColumnHeader _columnHeader;
        private UltraGridGroupByRow _groupByRow;

        public Form1()
        {
            this.InitializeComponent();
            this.Load += Form1_Load;
            this.ugUsers.MouseUp += UgUsers_MouseUp;
        }

        private void Form1_Load(object sender, System.EventArgs e)
        {
            var data = this.GetData(100, 5);
            this.ugUsers.SetDataBinding(data, data.TableName);
            this.ugUsers.DisplayLayout.Bands[0].SortedColumns.Add(this.ugUsers.DisplayLayout.Bands[0].Columns[1], false, true);
            this.ugUsers.Rows.ExpandAll(true);
        }

        private void UgUsers_MouseUp(object sender, MouseEventArgs e)
        {
            UltraGrid grid = (UltraGrid)sender;
            if(grid == null)
                return;

            if(e.Button != MouseButtons.Right)
            { return; }

            _uiElement = grid.DisplayLayout.UIElement.LastElementEntered;
            _columnHeader = _uiElement.GetContext(typeof(Infragistics.Win.UltraWinGrid.ColumnHeader)) as Infragistics.Win.UltraWinGrid.ColumnHeader;

            if(_columnHeader != null)
            {
                if(_columnHeader.Column.Key != "Selected")
                { return; }

                //  Get the BandHeadersUIElement. This element contains all the column headers.
                //  We will compare this element Rect.Top with the GroupByRowConnectorUIElement
                //  Rect.Top to find the row connector which matches the group by we need
                var bandHeaderElement = _uiElement.GetAncestor(typeof(BandHeadersUIElement));

                //  Get the RowColRegionIntersectionUIElement. This element contains all the elements
                //  of the grid in this region
                var rowColRegionElement = _uiElement.GetAncestor(typeof(RowColRegionIntersectionUIElement));

                //  Go through all the child elements of RowColRegionIntersectionUIElement. If the
                //  child element is GroupByRowConnectorUIElement compare its Top with the Top of
                //  BandHeadersUIElement.
                foreach(var element in rowColRegionElement.ChildElements)
                {
                    if(element is GroupByRowConnectorUIElement)
                    {
                        if(element.Rect.Top == bandHeaderElement.Rect.Top)
                        {
                            //  Get the UltraGridGroupByRow related to this connector element and exit the loop
                            _groupByRow = element.GetContext() as UltraGridGroupByRow;
                            if(_groupByRow != null)
                                break;
                        }
                    }
                }

                var cmContextMenu = new ContextMenu();
                cmContextMenu.MenuItems.Add("Select All", SelectAll_Click);
                cmContextMenu.MenuItems.Add("Unselect All", UnselectAll_Click);
                cmContextMenu.Show(ugUsers, e.Location);
            }
        }

        private void SelectAll_Click(object sender, EventArgs e)
        {
            UltraGridRow[] rows = null;

            //  if we have group by row get its rows, otherwise get all the rows
            if(_groupByRow != null)
                rows = (UltraGridRow[])_groupByRow.Rows.All;
            else
                rows = (UltraGridRow[])this.ugUsers.Rows.All;

            this.ugUsers.Selected.Rows.Clear();
            this.ugUsers.Selected.Rows.AddRange(rows);
            _groupByRow = null;
        }

        private void UnselectAll_Click(object sender, EventArgs e)
        {
            this.ugUsers.Selected.Rows.Clear();
        }

        private DataTable GetData(int rows = 20, int cols = 5)
        {
            DataTable dt = new DataTable();

            for(int col = 0; col < cols; col++)
            {
                if(col == 0)
                {
                    dt.Columns.Add("Selected", typeof(int));
                    continue;
                }

                dt.Columns.Add(string.Format("Column #{0}", col + 1), typeof(int));
            }

            var random = new Random();
            for(int row = 0; row < rows; row++)
            {
                DataRow addRow = dt.NewRow();
                foreach(DataColumn column in dt.Columns)
                {
                    addRow[column] = random.Next(10);
                }

                dt.Rows.Add(addRow);
            }

            return dt;
        }
    }
}
